from ..._common.const import (
    FinancialDataComponentType as _FinancialDataComponentType,
)
from .._req_builder._list import _list_dataitem_financial
from ..._prismcomponent.datacomponent import (
    _BalanceSheet,
    _IncomeStatement,
    _CashFlow,
    _OtherFinancial,
)
from ..._utils.validate_utils import _validate_args


__all__ = [
    'balance_sheet',
    'income_statement',
    'cash_flow',
    'dps',
    'eps',
    'other',
    'dataitems',
    'balance_sheet_dataitems',
    'income_statement_dataitems',
    'cash_flow_dataitems',
    'other_dataitems',
]


@_validate_args
def _build_financial(
    datacomponentclass,
    dataitemid: int,
    periodtype: str,
    currency: str = None,
    preliminary: bool = True,
    package: str = "CIQ Premium Financials",
):
    if (datacomponentclass is _BalanceSheet) and (periodtype in ['YTD', 'LTM']):
        raise SystemExit(f'Balance Sheet cannot take {periodtype} as periodtype.')
    return datacomponentclass(
        dataitemid=dataitemid,
        periodtype=periodtype,
        package=package,
        preliminary=preliminary,
        currency=currency,
    )


def balance_sheet(*args, **kwargs):
    return _build_financial(_BalanceSheet, *args, **kwargs)


def cash_flow(*args, **kwargs):
    return _build_financial(_CashFlow, *args, **kwargs)


def dps(*args, **kwargs):
    return _build_financial(_IncomeStatement, *args, **kwargs)


def eps(*args, **kwargs):
    return _build_financial(_IncomeStatement, *args, **kwargs)


def income_statement(*args, **kwargs):
    return _build_financial(_IncomeStatement, *args, **kwargs)


def other(*args, **kwargs):
    return _build_financial(_OtherFinancial, *args, **kwargs)


@_validate_args
def dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(None, search, package)


@_validate_args
def balance_sheet_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.BALANCE_SHEET, search, package)


@_validate_args
def income_statement_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.INCOME_STATEMENT, search, package)


@_validate_args
def cash_flow_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.CASH_FLOW, search, package)


@_validate_args
def other_dataitems(search: str = None, package: str = None):
    return _list_dataitem_financial(_FinancialDataComponentType.OTHER_FINANCIAL, search, package)
