from ..._common.const import (
    MarketDataComponentType as _MarketDataComponentType,
)
from .._req_builder._list import _list_dataitem_market
from ..._prismcomponent.datacomponent import (
    _Open,
    _Close,
    _High,
    _Low,
    _Bid,
    _Ask,
    _VWAP,
    _MarketCap,
    _Volume,
    _Dividend,
    _DividendAdjustmentFactor,
    _Split,
    _SplitAdjustmentFactor,
    _ExchangeRate,
    _ShortInterest,
)
from ..._utils.validate_utils import _validate_args


__all__ = [
    'open',
    'close',
    'high',
    'low',
    'bid',
    'ask',
    'vwap',
    'market_cap',
    'volume',
    'dividend',
    'exchange_rate',
    'short_interest',
    'split',
    'split_adjustment_factor'
    'dataitems',
]

# TODO: adjustement bool validation

@_validate_args
def _build_price(
    datacomponentclass,
    adjustment: str = "All",
    currency: str = None,
    package: str = None,
):
    return datacomponentclass(
        package=package,
        adjustment=adjustment,
        currency=currency,
    )


def open(*args, **kwargs):
    return _build_price(_Open, *args, **kwargs)


def close(*args, **kwargs):
    return _build_price(_Close, *args, **kwargs)


def high(*args, **kwargs):
    return _build_price(_High, *args, **kwargs)


def low(*args, **kwargs):
    return _build_price(_Low, *args, **kwargs)


def ask(*args, **kwargs):
    return _build_price(_Ask, *args, **kwargs)


def bid(*args, **kwargs):
    return _build_price(_Bid, *args, **kwargs)


def vwap(*args, **kwargs):
    return _build_price(_VWAP, *args, **kwargs)


@_validate_args
def market_cap(currency: str = None, package: str = None):
    return _MarketCap(currency=currency, package=package)


@_validate_args
def volume(adjustment: bool = True, package: str = None):
    return _Volume(adjustment=adjustment, package=package)


@_validate_args
def dividend(adjustment: bool = True, currency: bool = None, package: str = None):
    return _Dividend(adjustment=adjustment, currency=currency, package=package)


@_validate_args
def dividend_adjustment_factor(adjustment: bool = True, currency: bool = None, package: str = None):
    return _DividendAdjustmentFactor(adjustment=adjustment, currency=currency, package=package)


@_validate_args
def split(package=None):
    return _Split(package=package)


@_validate_args
def split_adjustment_factor(adjustment: bool = True, currency: bool = None, package: str = None):
    return _SplitAdjustmentFactor(adjustment=adjustment, currency=currency, package=package)


@_validate_args
def exchange_rate(currency: list, to_convert: bool = False):
    return _ExchangeRate(currency=currency, to_convert=to_convert)


@_validate_args
def short_interest(dataitemid: int, package: str = None):
    return _ShortInterest(dataitemid=dataitemid, package=package)


@_validate_args
def dataitems(search: str = None, package: str = None):
    return _list_dataitem_market(None, search, package)


@_validate_args
def short_interest_dataitems(search: str = None, package: str = None):
    return _list_dataitem_market(
        _MarketDataComponentType.SHORT_INTEREST, search, package
    )
