from ..._common.config import URL_PREFERENCES as _URL_PREFERENCES
from ..._common.const import COMPONENT2CATEGORY as _COMPONENT2CATEGORY
from ..._utils import get as _get, post as _post, patch as _patch, _validate_args


__all__ = ['set_default_attributes', 'get_default_attributes', 'set_package_priority', 'get_package_priority']


@_validate_args
def set_default_attributes(attributes: list):
    ret = _patch(_URL_PREFERENCES + '/attribute', {'attributes': attributes}, None)
    print(f'{ret["message"]}')
    print(f'Default attributes are: {get_default_attributes()}')
    return ret


@_validate_args
def get_default_attributes():
    ret = _get(_URL_PREFERENCES + '/attribute')
    return ret['attributes']


@_validate_args
def set_package_priority(datacomponent: str, package_priority: list):
    if datacomponent not in _COMPONENT2CATEGORY.keys():
        raise SystemExit('Datacomponent should be one of prism data components.')
    body = []
    for i in package_priority:
        if not isinstance(i, dict):
            raise SystemExit('Elements of pacakge priority should be a dictionary.')
        if {'package', 'priority'} != set(i.keys()):
            raise SystemExit('Package priority should have packagename and priority as keys.')
        body.append({'datacomponent': datacomponent, 'packagename': i['package'], 'priority': i['priority']})

    ret = _patch(_URL_PREFERENCES + '/package_priority', None, body)
    print(f'{ret["message"]}')
    return ret


@_validate_args
def get_package_priority():
    return _get(_URL_PREFERENCES + '/package_priority')
