import os
import errno
import sys
import requests
import uuid

from .._common.const import PACKAGE_NAME
from .._common.config import URL_WEB_AUTH, HEADERS


def _authentication():
    package_path = _find_file_path(PACKAGE_NAME)
    try:
        headers = HEADERS.copy()
        with open(package_path + '/token.txt', 'r') as f:
            token = f.readline()
        headers.update({'Authorization': 'Bearer {}'.format(token)})
        req_id = str(uuid.uuid4())[:8]
        headers.update({'requestid': req_id})
    except FileNotFoundError:
        raise ValueError(f'\033[91mNot Authenticated\033[0m: Please Login First', True, "Custom") from None
    return headers


def _create_token(response: requests.models.Response, username: str):
    package_path = _find_file_path(PACKAGE_NAME)
    token = response.json()['access_token']
    with open(package_path + '/token.txt', 'w') as f:
        f.write(token)
    return


def _find_file_path(file_name: str):
    sys.path.append(os.getcwd())
    for path in sys.path:
        if os.path.exists(path + '/' + file_name):
            return path + '/' + file_name
    raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), file_name)


class TokenDoesNotExistError(Exception):
    def __init__(self):
        super().__init__('You should first login to create a token')


def _get_web_authentication_token():
    headers = _authentication()
    res = requests.post(URL_WEB_AUTH, headers=headers)
    web_auth_token = res.json().get('access_token')
    return web_auth_token