import copy
import shutil
from enum import Enum
from functools import wraps
from pathlib import Path

from .._common.const import COMPONENT2CATEGORY, FUNCTIONS

def _req_call(call_type):
    def decorator(func):
        @wraps(func)
        def wrapper(obj, *args, **kwargs):
            func(obj, *args, **kwargs)
            vars = func.__code__.co_varnames[1:]
            params = dict(zip(vars[: len(args)], args))
            params.update(dict(kwargs))
            fn = getattr(call_type, func.__name__)
            return fn(obj, **params)

        return wrapper

    return decorator


def update_query(component):
    print("Please Modify the below code to update the component. - Don't forget to save it.")
    return print_query_code(component._query, 'a0', return_query_var=False)


def print_query_code(component_query, query_var, return_query_var=True):
    component_type = component_query['component_type']
    component_name = component_query['component_name']
    component_args = copy.deepcopy(component_query['component_args'])
    children = component_query['children']
    print_str = ''
    if component_type == 'datacomponent':
        if not component_name == 'PrismValue':
            category_component = COMPONENT2CATEGORY[component_name]
            print_str += (
                f'prism.{category_component["category"]}.{category_component["component"]}({print_args(component_args)})'
            )
    elif component_type == 'taskcomponent':
        for idx, i in enumerate(list(component_args.keys())):
            if i.endswith('_dataquery'):
                arg = component_args.pop(i)
                child_print_str, qv = print_query_code(arg, query_var)
                if arg['component_type'] == 'datacomponent':
                    print_str += f'{qv} = {child_print_str}\n'
                else:
                    print_str += f'{child_print_str}\n'
                component_args[i] = f'^*{qv}'
                query_var = f'{qv[0]}{int(qv[1:])+1}'
        print_str += f'prism.{component_name}({print_args(component_args)})'
    elif component_type == 'functioncomponent':
        fn_name = FUNCTIONS.get(component_name)
        fn_str = ''
        if component_name == 'map':
            for idx, i in enumerate(list(component_args.keys())):
                if component_args[i]['component_name'] != 'PrismValue':
                    arg = component_args.pop(i)
                    child_print_str, qv = print_query_code(arg, query_var)
                    if arg['component_type'] == 'datacomponent':
                        print_str += f'{qv} = {child_print_str}\n'
                    else:
                        print_str += f'{child_print_str}\n'
                    component_args[i] = qv
                    query_var = f'{qv[0]}{int(qv[1:])+1}'
                else:
                    component_args[i] = component_args[i]['component_args']['data']
        for idx, i in enumerate(children):
            if component_name == 'map':
                if idx != 0:
                    break
            child_print_str, qv = print_query_code(i, query_var)
            if i['component_type'] == 'datacomponent':
                print_str += f'{qv} = {child_print_str}\n'
            else:
                print_str += f'{child_print_str}\n'
            if fn_name is None:
                if component_name == 'map':
                    fn_str += f'{qv}.{component_name}({component_args})'
                else:
                    fn_str += f'{qv}.{component_name}({print_args(component_args)})'
            else:
                if i['component_name'] == 'PrismValue':
                    c = i['component_args']['data']
                else:
                    c = qv
                if idx != len(children) - 1:
                    fn_str += f'{c} {fn_name["op"]}'
                else:
                    fn_str += f' {c}'
            if i['component_name'] != 'PrismValue':
                query_var = f'{qv[0]}{int(qv[1:])+1}'
        print_str += f'{query_var} = {fn_str}'
    if return_query_var:
        return print_str, query_var
    else:
        return print_str


def print_args(component_args):
    print_str = ''
    for idx, (k, v) in enumerate(component_args.items()):
        print_str += str(k)
        print_str += '='
        if isinstance(v, str):
            if v.startswith('^*'):
                print_str += v.split('^*')[-1]
            else:
                print_str += f'"{v}"'
        elif isinstance(v, Enum):
            print_str += f'"{v.value}"'
        else:
            print_str += str(v)
        if idx != len(component_args) - 1:
            print_str += ', '
    return print_str


class DisplayablePath(object):
    display_filename_prefix_middle = '├──'
    display_filename_prefix_last = '└──'
    display_parent_prefix_middle = '    '
    display_parent_prefix_last = '│   '

    def __init__(self, path, parent_path, is_last):
        self.path = Path(str(path))
        self.parent = parent_path
        self.is_last = is_last
        if self.parent:
            self.depth = self.parent.depth + 1
        else:
            self.depth = 0

    @property
    def displayname(self):
        if self.path.is_dir():
            return self.path.name + '/'
        return self.path.name

    @classmethod
    def make_tree(cls, root, parent=None, is_last=False, criteria=None):
        root = Path(str(root))
        criteria = criteria or cls._default_criteria

        displayable_root = cls(root, parent, is_last)
        yield displayable_root

        children = sorted(list(path
                               for path in root.iterdir()
                               if criteria(path)),
                          key=lambda s: str(s).lower())
        count = 1
        for path in children:
            is_last = count == len(children)
            if path.is_dir():
                yield from cls.make_tree(path,
                                         parent=displayable_root,
                                         is_last=is_last,
                                         criteria=criteria)
            else:
                yield cls(path, displayable_root, is_last)
            count += 1

    @classmethod
    def _default_criteria(cls, path):
        return True

    @property
    def displayname(self):
        if self.path.is_dir():
            return self.path.name + '/'
        return self.path.name

    def displayable(self):
        if self.parent is None:
            return self.displayname

        _filename_prefix = (self.display_filename_prefix_last
                            if self.is_last
                            else self.display_filename_prefix_middle)

        parts = ['{!s} {!s}'.format(_filename_prefix,
                                    self.displayname)]

        parent = self.parent
        while parent and parent.parent is not None:
            parts.append(self.display_parent_prefix_middle
                         if parent.is_last
                         else self.display_parent_prefix_last)
            parent = parent.parent

        return ''.join(reversed(parts))


def create_files(paths, directory):
    for path in paths:
        output_file = Path(f'./tmp/{directory}/' + path)
        output_file.parent.mkdir(exist_ok=True, parents=True)
        output_file.write_text("FOOBAR")


def plot_tree(paths: list, directory: str):
    # Create mock files on local file system
    rootdir = f'./tmp/{directory}'
    create_files(paths, directory)

    if not paths:
        return
    # Remove username in the front
    # Display all the files
    paths = DisplayablePath.make_tree(Path(rootdir))
    for path in paths:
        print(path.displayable())

    # delete folder
    shutil.rmtree(rootdir)
    return

