from enum import Enum
import glob
import orjson
import os
import pandas as pd
import pyarrow as pa
import py7zr
import requests
import shutil

import prism
from .._utils import _authentication
from .._common.config import *
from .._common.const import *


def _process_response(res, content=None):
    if res.status_code >= 400:

        if res.status_code == 401:
            raise ValueError(f'\033[91mNot Authenticated\033[0m: Please Login First', True, "Custom") from None

        if res.status_code == 422:
            arg = res.json()['detail'][0]['loc'][2]
            list = res.json()['detail'][0]['ctx']['enum_values']
            raise ValueError(f'Value at "{arg}" is not a valid enumeration member', True, list) from None

    if res.status_code >= 400:
        # if res.status_code == 422:
        #     try:
        #         msg = res.json()['detail'][0]['msg']
        #         type = res.json()['detail'][0]['type']
        #         list = res.json()['detail'][0]['ctx']['enum_values']
        #     except Exception as e:
        #         raise ValueError(e, True, list) from None
        # else:
        try:
            err_msg = res.json()['message']
        except Exception as e:
            if content:
                err_msg = content
            else:
                err_msg = res.content
        # print(res.json())
        raise SystemExit(err_msg)
    result = None
    if content:
        resjson = orjson.loads(content)
    else:
        resjson = orjson.loads(res.content)
    restype = resjson['restype']
    rescontent = resjson['rescontent']
    if restype == 'status':
        result = rescontent
    else:
        data = rescontent.get('data')
        if data is not None:
            if ('columns' in data.keys()) and (('data' in data.keys())):
                cols = data.get('columns')
                data = data.get('data')
                datecolumns = []
                cols_to_use = []
                for c in cols:
                    if c['dtype'] == 'datetime64[ns]':
                        datecolumns.append(c['column'])
                    cols_to_use.append(c['column'])
                result = pd.DataFrame(data, columns=cols_to_use)
                for c in datecolumns:
                    result[c] = pd.to_datetime(result[c], unit='ns')
            else:
                result = data
        else:
            result = rescontent
    return result


def _process_fileresponse(res, res_data_type, content=None):
    if res.status_code >= 400:
        try:
            err_msg = res.json()['message']
        except:
            err_msg = res.content
        raise SystemExit(err_msg)
    try:
        if not os.path.isdir('tmp/' + res_data_type + '/zip/'):
            os.makedirs('tmp/' + res_data_type + '/zip/')
        if not os.path.isdir('tmp/' + res_data_type + '/unzip/'):
            os.makedirs('tmp/' + res_data_type + '/unzip/')
        if content:
            open('tmp/' + res_data_type + '/zip/temp_result.7z', 'wb').write(content)
        else:
            open('tmp/' + res_data_type + '/zip/temp_result.7z', 'wb').write(res.content)
        with py7zr.SevenZipFile('tmp/' + res_data_type + '/zip/temp_result.7z', 'r') as archive:
            archive.extractall(path='tmp/' + res_data_type + '/unzip')
        dfs = {}
        for f in sorted(glob.glob('tmp/' + res_data_type + '/unzip/*.arrow')):
            filename = f.replace('\\', '/')
            filename = filename.split('/')[-1].split('.')[0]
            with pa.OSFile(f, 'rb') as source:
                dfs[filename] = pa.ipc.open_file(source).read_pandas()
    except:
        raise SystemExit('Extension response processing error')

    finally:
        shutil.rmtree('tmp/' + res_data_type)
    if len(dfs) == 1:
        return list(dfs.values())[0]
    if res_data_type == 'data':
        return list(dfs.values())
    else:
        return dfs


def get(url, params=None, stream=False):
    headers = _authentication()
    res = requests.get(url=url, params=params, headers=headers, stream=stream)
    return _process_response(res)


def post(url, params, body, stream=False):
    headers = _authentication()
    res = requests.post(url=url, params=params, json=body, headers=headers, stream=stream)
    return _process_response(res)


def patch(url, params, body):
    headers = _authentication()
    res = requests.patch(url=url, params=params, json=body, headers=headers)
    return _process_response(res)


def delete(url, params=None):
    headers = _authentication()
    res = requests.delete(url=url, params=params, headers=headers)
    return _process_response(res)


def _fetch_and_parse(url, ext, search=None):
    df = get(url, {'search': search})
    if not df.empty:
        directorytype = FILEEXTENSION[ext]
        df['path'] = df['path'].str.split(prism.username + '/' + directorytype + '/', expand=True).iloc[:, -1]
        df['path'] = df['path'].str.split('.' + ext, expand=True).iloc[:, 0]
        df = df.rename(columns={'path': directorytype + 'name'})
    return df
