from functools import wraps
import inspect
from multiprocessing.sharedctypes import Value
import re
import sys

import pandas as pd
from .._common.const import *

import traceback
import IPython

def change_function(func):
    @wraps(func)
    def showtraceback(*args, **kwargs):
        # extract exception type, value and traceback
        etype, evalue, tb = sys.exc_info()

        l = traceback.format_exception(*sys.exc_info())

        # If exception occurs in package prism
        if len(evalue.args) > 1 :
            try:
                default_e = evalue.args[0]
                prism_err = evalue.args[1]
                extra_parameter = evalue.args[2]
            except:
                value = func(*args, **kwargs)
                return value

            if prism_err:
                try:

                    if type(extra_parameter).__name__ == 'EnumMeta' :
                        print(f'\033[91m{etype.__name__}\033[0m: {default_e}')
                        print(f'\033[92mValid Arguments\033[0m: {[item.value for item in extra_parameter]}')
                    elif type(extra_parameter).__name__ == 'list' :
                        print(f'\033[91m{etype.__name__}\033[0m: {default_e}')
                        print(f'\033[92mValid Arguments\033[0m: {[item for item in extra_parameter]}')
                    elif extra_parameter == None:
                        print(f'\033[91m{etype.__name__}\033[0m: {default_e}')
                        return
                    elif extra_parameter == 'Custom':
                        print(default_e)
                        return
                    else:
                        print(f'\033[91m{etype.__name__}\033[0m: {default_e}')
                        print(f'\033[92mValid Arguments\033[0m: BlahBlahBlah')
                except:
                    value = func(*args, **kwargs)
                    return value
        else:
            # otherwise run the original hook
            value = func(*args, **kwargs)
            return value
    return showtraceback

def handle_exception(exc_type, exc_value, exc_traceback):

    if len(exc_value.args) > 1:
        try:
            default_e = exc_value.args[0]
            prism_err = exc_value.args[1]
            extra_parameter = exc_value.args[2]
        except:
            return sys.__excepthook__(exc_type, exc_value, exc_traceback)

        if prism_err:
            try:
                print(f'\033[91m{exc_type.__name__}\033[0m: {default_e}')
                print(f'\033[92mValid Arguments\033[0m: {[item.value for item in extra_parameter]}')
            except:
                return sys.__excepthook__(exc_type, exc_value, exc_traceback) 
    else:
        return sys.__excepthook__(exc_type, exc_value, exc_traceback) 


IPython.core.interactiveshell.InteractiveShell.showtraceback = change_function(IPython.core.interactiveshell.InteractiveShell.showtraceback)
sys.excepthook = handle_exception

def _validate_args(function):
    @wraps(function)
    def type_checker(*args, **kwargs):
        func_params_dict = inspect.signature(function).parameters

        # construct param-arg pairs: order matters - do not use set operations
        pos_params = tuple(p for p in func_params_dict.keys() if p not in kwargs.keys())
        if 'self' in pos_params:
            pos_params = tuple(p for p in pos_params if p != 'self')
            pos_args = dict(zip(pos_params, args[1:]))
        else:
            pos_args = dict(zip(pos_params, args))
        all_args = kwargs.copy()
        all_args.update(pos_args)

        args = list(args)
        for param, v in func_params_dict.items():

            # if None, not provided (nor required)
            arg = all_args.get(param, v.default)
            if (arg is not None) and (arg is not inspect._empty):
                # check types
                param_type = v.annotation  # if empty, no hints
                if param_type is not inspect._empty:
                    if not issubclass(type(arg), param_type):
                        raise TypeError(f'Type of {param} is {type(arg)} and not {param_type}')

                # validation rules
                arg_enum = None
                if param == 'frequency':
                    try:
                        arg_enum = FrequencyType(arg)
                    except Exception as e:
                        raise ValueError(e, True, FrequencyType) from None
                elif param == 'datetype':
                    try:
                        arg_enum = DateType(arg)
                    except Exception as e:
                        raise ValueError(e, True, DateType) from None
                elif param == 'periodtype':
                    try:
                        arg_enum = PeriodType(arg)
                    except Exception as e:
                        raise ValueError(e, True, PeriodType) from None
                elif param == 'adjustment':
                    try:
                        arg_enum = AdjustmentType(arg)
                    except Exception as e:
                        raise ValueError(e, True, AdjustmentType) from None
                elif param == 'shownid':
                    try:
                        arg_enum = [SMValueType(id) for id in arg]
                    except Exception as e:
                        raise ValueError(e, True, SMValueType) from None
                elif param == 'rank_method':
                    try:
                        arg_enum = RankType(arg)
                    except Exception as e:
                        raise ValueError(e, True, RankType) from None
                elif param in ['startdate', 'enddate']:
                    try:
                        date = pd.to_datetime(arg)
                    except Exception as e:
                        raise ValueError(f'Unknown string format. Cannot parse "{arg}" to a date', True, None) from None
                        
                    assert (date >= BEGINNING_DATE) & (date <= ACTIVE_DATE), 'Not a valid date.'

                elif 'universename' in param:
                    regex = re.compile('[@_!#$%^&*()<>?/\|}{~:]`"\'')
                    if isinstance(arg, list):
                        for a in arg:
                            if regex.search(a) is not None:
                                raise TypeError('universename not to include special characters')
                    else:
                        if regex.search(arg) is not None:
                            raise TypeError('universename not to include special characters')
                elif (function.__name__ == 'log') & (param == 'base'):
                    if arg <= 0:
                        raise Exception("base condition error")

                if arg_enum is not None:
                    if param in pos_args.keys():
                        for i, k in enumerate(func_params_dict.keys()):
                            if param == k:
                                break
                        args[i] = arg_enum
                    else:
                        kwargs[param] = arg_enum

        return function(*args, **kwargs)

    return type_checker
